/* Mark de Jong Electronics (c) 2005
*
* Project: S65 Display
*
* Filename: S65_display.c
*
*
*/
#include "S65_display.h"

// private data

static struct
{
  unsigned int display_type;
  unsigned int display_state;
} s65;


// sequences for S65_LS20
// Controller LR38826?
unsigned int s65_ls20_init1[] = { 0xFDFD, 0xFDFD };
unsigned int s65_ls20_init2[] = { 0xEF00, 0xEE04, 0x1B04, 0xFEFE, 0xFEFE,
                                  0xEF90, 0x4A04, 0x7F3F, 0xEE04, 0x4306 };
unsigned int s65_ls20_init3[] = { 0xEF90, 0x0983, 0x0800, 0x0BAF, 0x0A00, 0x0500, 0x0600, 0x0700,
                                  0xEF00,
                                  0xEE0C,
                                  0xEF90, 0x0080,
                                  0xEFB0, 0x4902,
                                  0xEF00, 0x7F01, 0xE181 };
unsigned int s65_ls20_init4[] = { 0xE202 };
unsigned int s65_ls20_init5[] = { 0xE276 };
unsigned int s65_ls20_init6[] = { 0xE183 };
unsigned int s65_ls20_init7[] = { 0x8001 };

// sequences for S65_LPH88
// Controller HD66773


// sequences for S65_L2F50
// Controller ??

unsigned int s65_display_init( unsigned int display_type )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  if( s65.display_type != S65_UNKOWN )
  {
    if( display_type != S65_DETECT )
    {
      s65.display_type = display_type;
    }
    else
    {
      // detection not yet possible, unkown
      s65.display_type = s65_display_detect_display_type();
    }
  }
  else
  {
    if( s65.display_type != display_type )
    {
      // error handling re-initialise with other display type requested.
    }
  }

  // setup IO-ports or SPI-interface
  s65_driver_if_init();

  // power-up display
  s65_driver_if_power( S65_POWER_ON );

  // release reset
  s65_driver_if_reset( S65_RESET_RELEASE );

  if( s65.display_type == S65_LS20 )
  {
    // send init sequence 1
    s65_driver_if_ls20_send_command_words( sizeof(s65_ls20_init1), (unsigned int *)&s65_ls20_init1 );

    // send init sequence 2
    s65_driver_if_ls20_send_command_words( sizeof(s65_ls20_init2), (unsigned int *)&s65_ls20_init2 );

    // send init sequence 3
    s65_driver_if_ls20_send_command_words( sizeof(s65_ls20_init3), (unsigned int *)&s65_ls20_init3 );

    // send init sequence 4
    s65_driver_if_ls20_send_command_words( sizeof(s65_ls20_init4), (unsigned int *)&s65_ls20_init4 );

    // send init sequence 5
    s65_driver_if_ls20_send_command_words( sizeof(s65_ls20_init5), (unsigned int *)&s65_ls20_init5 );

    // send init sequence 6
    s65_driver_if_ls20_send_command_words( sizeof(s65_ls20_init6), (unsigned int *)&s65_ls20_init6 );

    // send init sequence 7
    s65_driver_if_ls20_send_command_words( sizeof(s65_ls20_init7), (unsigned int *)&s65_ls20_init7 );
  }
  else if( s65.display_type == S65_LPH88 )
  {
    // Start Oscillation
    s65_driver_if_lph88_send_command_word( 0x0000, 0x0001 );

    // Display Control
    s65_driver_if_lph88_send_command_word( 0x0007, 0x0000 );
    // LCD drive AC control
    s65_driver_if_lph88_send_command_word( 0x0002, 0x0400 );
    // Power control(2)
    s65_driver_if_lph88_send_command_word( 0x0004, 0x0000 );
    // Power control(3)
    s65_driver_if_lph88_send_command_word( 0x000C, 0x0001 );
    // Power control(4)
    s65_driver_if_lph88_send_command_word( 0x000D, 0x0616 );
    // Power control(5)
    s65_driver_if_lph88_send_command_word( 0x000E, 0x1010 );

    // Power control(1)
    s65_driver_if_lph88_send_command_word( 0x0003, 0x000C );
    // Power control(5)
    s65_driver_if_lph88_send_command_word( 0x000E, 0x2D1F );

    // Power control(4)
    s65_driver_if_lph88_send_command_word( 0x000D, 0x0616 );

    // Driver output control
    s65_driver_if_lph88_send_command_word( 0x0001, 0x0215 );
    // Gate scan starting position
    s65_driver_if_lph88_send_command_word( 0x000F, 0x0000 );
    // Frame cycle control
    s65_driver_if_lph88_send_command_word( 0x000B, 0x0000 );
    // Vertical scroll control
    s65_driver_if_lph88_send_command_word( 0x0011, 0x0000 );
    // Compare register
    s65_driver_if_lph88_send_command_word( 0x0006, 0x0000 );
    // Entry mode
    s65_driver_if_lph88_send_command_word( 0x0005, 0x0030 );
    // First display drive position
    s65_driver_if_lph88_send_command_word( 0x0014, 0xAF00 );
    // Second display drive position
    s65_driver_if_lph88_send_command_word( 0x0015, 0x0000 );
    // Horizontal RAM address position
    s65_driver_if_lph88_send_command_word( 0x0016, 0x8300 );
    // Vertical RAM address position
    s65_driver_if_lph88_send_command_word( 0x0016, 0xAF00 );
    // RAM data write
    s65_driver_if_lph88_send_command_word( 0x0020, 0x0000 );
    // y control(1)
    s65_driver_if_lph88_send_command_word( 0x0030, 0x0000 );
    // y control(2)
    s65_driver_if_lph88_send_command_word( 0x0031, 0x0400 );
    // y control(3)
    s65_driver_if_lph88_send_command_word( 0x0032, 0x0207 );
    // y control(4)
    s65_driver_if_lph88_send_command_word( 0x0033, 0x0700 );
    // y control(5)
    s65_driver_if_lph88_send_command_word( 0x0034, 0x0005 );
    // y control(6)
    s65_driver_if_lph88_send_command_word( 0x0035, 0x0703 );
    // y control(7)
    s65_driver_if_lph88_send_command_word( 0x0036, 0x0707 );
    // y control(8)
    s65_driver_if_lph88_send_command_word( 0x0037, 0x0007 );
    // y control(9)
    s65_driver_if_lph88_send_command_word( 0x003A, 0x1200 );
    // y control(10)
    s65_driver_if_lph88_send_command_word( 0x003B, 0x0009 );

    // Display Control
    s65_driver_if_lph88_send_command_word( 0x0007, 0x0005 );

    // Display Control
    s65_driver_if_lph88_send_command_word( 0x0007, 0x0025 );

    // Display Control
    s65_driver_if_lph88_send_command_word( 0x0007, 0x0027 );

    // Display Control
    s65_driver_if_lph88_send_command_word( 0x0007, 0x0037 );

  }
  else //S65_L2F50
  {
  }

  return( ret_value );
}

unsigned int s65_display_detect_display_type( void )
{
  s65.display_type = S65_LS20;

  return( s65.display_type );
}

unsigned int s65_display_get_display_type( void )
{
  return( s65.display_type );
}

unsigned int s65_display_power( unsigned int power_state )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  return( ret_value );
}

unsigned int s65_display_backlight( unsigned int backlight_state, unsigned char backlight_value )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  if( (backlight_state != S65_DISPLAY_BL_ON) && (backlight_state != S65_DISPLAY_BL_ON) )
  {
    ret_value = S65_ERROR;
  }
  else
  {
      if( backlight_value > S65_DISPLAY_BL_MAX )
      {
        backlight_value = S65_DISPLAY_BL_MAX;
      }
      //set backlight brightness

      //set backlight off/on
  }
  return( ret_value );
}

unsigned int s65_display_clear_screen( void )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  s65_display_fill_screen( COLOR_WHITE );

  return( ret_value );
}

unsigned int s65_display_fill_screen( unsigned int color )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  return( ret_value );
}

