/* Mark de Jong Electronics (c) 2005
*
* Project: S65 Display
*
* Filename: S65_driver_if.c
*
*
*/
#include "S65_driver_cfg.h"
#include "S65_driver_if.h"

// public functions
unsigned int s65_driver_if_init( void )
{
  unsigned int ret_value;

  ret_value = S65_OK;

#ifdef S65_USE_IO_PORTS
  // set start-up levels
  io_reset_io( S65_POWER );
  io_reset_io( S65_IO_RESET );

  io_set_io( S65_IO_CS );
  io_reset_io( S65_IO_DATA );
  io_reset_io( S65_IO_CLK );
  io_set_io( S65_IO_RS );

  // set port directions
  io_set_dir_out( S65_POWER );
  io_set_dir_out( S65_IO_RESET );

  io_set_dir_out( S65_IO_CS );
  io_set_dir_out( S65_IO_DATA );
  io_set_dir_out( S65_IO_CLK );
  io_set_dir_out( S65_IO_RS );

  ret_value = S65_OK;
#else

#endif
  return( ret_value );
}


unsigned int s65_driver_if_reset( unsigned int reset_state )
{
  unsigned int ret_value;

  ret_value = S65_OK;
#ifdef S65_USE_IO_PORTS
  if( reset_state == S65_RESET_ACTIVE )
  {
    // activate reset
    io_reset_io( S65_IO_RESET );
  }
  else
  {
    // release reset
    io_set_io( S65_IO_RESET );
  }
#else

#endif
  return( ret_value );
}


unsigned int s65_driver_if_power( unsigned int power_state )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  // setup IO-ports or SPI-interface
#ifdef S65_USE_IO_PORTS_MSP430
  if( power_state == S65_POWER_ON )
  {
    // switch power on
    io_set_io( S65_POWER );
  }
  else
  {
    // switch power on
    io_set_io( S65_POWER );
  }
#else

#endif
  return( ret_value );
}


unsigned int s65_driver_if_backlight( unsigned int backlight_state, unsigned int backlight_value )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  // setup IO-ports or SPI-interface
#ifdef S65_USE_IO_PORTS

#else

#endif
  return( ret_value );
}

//S65_LS20
unsigned int s65_driver_if_ls20_send_command_words( unsigned int number, unsigned int *command_words )
{
  unsigned int ret_value;
  unsigned int word_counter;

  ret_value = S65_OK;

  // select command register and select display
  io_set_io( S65_IO_RS );
  io_reset_io( S65_IO_CLK );
  io_reset_io( S65_IO_CS );

  //send commands
  for( word_counter=0; word_counter<number; word_counter++ )
  {
    s65_driver_send_word( *command_words );
    command_words++;
  }

  // deselect display
  io_set_io( S65_IO_CS );

  return( ret_value );
}


unsigned int s65_driver_if_ls20_send_data_words( unsigned int number, unsigned int *data_words )
{
  unsigned int ret_value;
  unsigned int word_counter;

  ret_value = S65_OK;

  // select data register and select display
  io_reset_io( S65_IO_RS );
  io_reset_io( S65_IO_CLK );
  io_reset_io( S65_IO_CS );

  //send commands
  for( word_counter=0; word_counter<number; word_counter++ )
  {
    s65_driver_send_word( *data_words );
    data_words++;
  }

  // deselect display
  io_set_io( S65_IO_CS );

  return( ret_value );
}

//S65_LPH88
unsigned int s65_driver_if_lph88_send_command_word( unsigned int command_index, unsigned int parameter )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  // select data register and select display
  io_reset_io( S65_IO_CLK );
  io_reset_io( S65_IO_CS );

  //send commands
  s65_driver_send_word( HD66773_COMMAND );
  s65_driver_send_word( command_index ); //select index
  s65_driver_send_word( parameter );   //send parameter

  // deselect display
  io_set_io( S65_IO_CS );
  return( ret_value );
}

unsigned int s65_driver_if_lph88_send_data_word( unsigned int data_word )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  // select data register and select display
  io_reset_io( S65_IO_CLK );
  io_reset_io( S65_IO_CS );

  //send commands
  s65_driver_send_word( HD66773_DATA );
  s65_driver_send_word( data_word );

  // deselect display
  io_set_io( S65_IO_CS );
  return( ret_value );
}

unsigned int s65_driver_if_lph88_send_data_n_word( unsigned int number, unsigned int data_words )
{
  unsigned int ret_value;
  unsigned int word_counter;

  ret_value = S65_OK;

  // select data register and select display
  io_reset_io( S65_IO_CLK );
  io_reset_io( S65_IO_CS );

  //send commands
  s65_driver_send_word( HD66773_DATA );
  for( word_counter=0; word_counter<number; word_counter++ )
  {
    s65_driver_send_word( data_words );
    data_words++;
  }

  // deselect display
  io_set_io( S65_IO_CS );
  return( ret_value );
}

unsigned int s65_driver_if_lph88_send_data_words( unsigned int number, unsigned int *data_words )
{
  unsigned int ret_value;
  unsigned int word_counter;

  ret_value = S65_OK;

  // select data register and select display
  io_reset_io( S65_IO_CLK );
  io_reset_io( S65_IO_CS );

  //send commands
  s65_driver_send_word( HD66773_DATA );
  for( word_counter=0; word_counter<number; word_counter++ )
  {
    s65_driver_send_word( *data_words );
    data_words++;
  }

  // deselect display
  io_set_io( S65_IO_CS );
  return( ret_value );
}

//S65_L2F50
unsigned int s65_driver_if_l2f50_send_command_words( unsigned int number, unsigned int *command_words )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  return( ret_value );
}

unsigned int s65_driver_if_l2f50_send_data_words( unsigned int number, unsigned int *data_words )
{
  unsigned int ret_value;

  ret_value = S65_OK;

  return( ret_value );
}

//S65 common
unsigned int s65_driver_send_word( unsigned int word )
{
  unsigned int ret_value;
  unsigned int bit_counter;

  ret_value = S65_OK;

#ifdef S65_USE_IO_PORTS
  //send one command
  for( bit_counter=0; bit_counter<16; bit_counter++ )
  {
    if( (word &(0x0001<<bit_counter)) == 0x01 )
    {
      io_set_io( S65_IO_DATA );
    }
    else
    {
      io_reset_io( S65_IO_DATA );
    }
    io_set_io( S65_IO_CLK );
    io_reset_io( S65_IO_CLK );
  }
#else

#endif
  return( ret_value );
}
